# Regime-dependent Effects of Uncertainty Shocks: A Structural Interpretation

Code to replicate "Regime-dependent Effects of Uncertainty Shocks: A Structural Interpretation", forthcoming at Quantitative Economics
by Stéphane LHUISSIER and Fabien TRIPIER

## Requirements

[Matlab](https://mathworks.com) and [Dynare](https://www.dynare.org/) 4.3.2 (you really need this version for the MS-DSGE!) and 4.6.1 (you need this version for the MS-SVAR). 

## MS-SVAR Estimation

Under Windows, assuming that you have installed Dynare in the standard location, type 'addpath C:\dynare\4.6.1\matlab\' to add the matlab subdirectory of your Dynare installation to MATLAB path.

Under macOS, assuming that you have installed Dynare in the standard location, type 'addpath /Applications/Dynare/4.6.1/matlab' to add the matlab subdirectory of your Dynare installation to MATLAB path.

Folder "msvar" allows to generate the results produced with the MS-SVAR model.

Skip steps (1) to (3) if you want to directly display the main results with the original files. 

- (1) To estimate and simulate the MS-SBVAR model with Sims and Zha (1998) prior, go to "PublicCodes_LT21\msvar\dynare" folder and type 'dynare Run_msVAR.mod' in MATLAB command window. Dynare will load the datafile called 'msbvar_data_62q3_18q2.mat', estimate and simulate (MCMC) the trivariate model with two independent MS processes: one MS process on shock variances and one MS on equation coefficients. See Dynare manual if you want to change some options.

NOTE: The estimation procedure may not converge directly to the best posterior mode at the first attempt since Dynare generates many draws from the distribution to initialize the maximization routine (csminwel). Please check that your mode is the same (or almost the same) as the one used in our paper. If you do not, please repeat the estimation procedure (or change Dynare's options). Look at the created file 'est_final_Run_msVAR.out' which reports the final posterior value. In our paper, we found that the best mode delivers a posterior value of 2186.28.

- (2) Run 'recover_draws_from_dynare.m' (in the folder "PublicCodes_LT21\msvar") to recover all draws from the posterior distributions of the benchmark model. The file 'load_draws_msVAR.mat' will be created. Skip this step if you want to run directly the original 'load draws msVAR.mat' file. 

- (3) Run 'recover_IRFs.m' (in the folder "PublicCodes_LT21\msvar") to compute impulse responses to an uncertainty shock under each state. The results will be stored in the 'IRFs_msbvar_draws.mat' file.

- (4) Run the following files (in the folder "PublicCodes_LT21\msvar") to report the main results:
    (a) Run 'gen_Fig1.m' to display smoothed probabilities (Figure 1)
    (b) Run 'gen_Fig2.m' to display the historical evolution of volatility for each variable (Figure 2) 
    (c) Run 'gen_Fig3.m' to generate impulse responses (Figure 3). If 'flag_dsge = 1', then the file will also store the necessary observables in 'MSVAR_IRFsAndSEs.mat' to estimate the MS-DSGE model.
    (d) Run 'gen_Tab1_and_Tab2.m' to report the standard deviations of shocks and the estimated transition matrices in a LateX format (Tables 1 and 2)

- (5) Folder "robustness" contains all models in the robustness section
        (a) 'gen_Alt60': 60% minimum contribution threshold
        (b) 'gen_Alt70': 70% minimum contribution threshold
        (c) 'gen_Alt80': 80% minimum contribution threshold
        (d) 'gen_h2': restrictions imposed over the next two quarters
        (e) 'gen_MaxVDs':  forecast error variance restrictions
 
## MS-DSGE Estimation

Under Windows, assuming that you have installed Dynare in the standard location, type 'addpath C:\dynare\4.3.2\matlab\' to add the matlab subdirectory of your Dynare installation to MATLAB path.

Under macOS, assuming that you have installed Dynare in the standard location, type 'addpath /Applications/Dynare/4.3.2/matlab' to add the matlab subdirectory of your Dynare installation to MATLAB path.

Folder "msdsge" allows to generate the results produced with the MS-DSGE model. 

Skip steps (1) to (2) if you want to directly display the main results with the original files.

- (1) To estimate and simulate (MCMC) the MS-DSGE model with a matching impulse responses approach, go to "PublicCodes_LT21\msdsge\estimation" folder and run 'run_mainfile.m'. Results will be stored in the "results" folder.

- (2) Several options can be modified
    (a) Line 19, choose the model you want to estimate: 
        'baseline': benchmark model
        'calvolwz': nominal rigidities calibrated from Liu, Waggoner and Zha (2011)
        'mplwz': Taylor ryle calibrated from Liu, Waggoner and Zha (2011)
        'nosticky': flexible prices and wages
    (b) Line 22 to 25, modify flags (0 or 1):
        'flag_fig': plot figure during estimation/simulation
        'flag_mode': find Posterior mode
        'flag_hess': compute Hessian at the mode
        'flag_rwmh': run MCMC

- (3) Go to "figures" or "tables" folders to generate the main results 
    (a) Run 'gen_Fig4.m' to display impulse responses (Figure 4)
    (b) Run 'gen_Fig5.m' to display counterfactual impulse responses for output (Figure 5)
    (c) Run 'gen_Fig6.m' to display counterfactual impulse responses for credit spread (Figure 6)
    (d) Run 'gen_Tab4.m' to display prior and posterior distributions of the baseline model in a LateX format (Table 4)
    (e) Run 'gen_Tab5.m' to display prior and posterior distributions of all models in a LateX format (Table 5)

## SOLVING MS-DSGE MODEL

Our micro-founded model is in line with Christiano, Motto, and Rostagno (CMR, 2014), but with two important differences, (i) we shut down all structural shocks, except for uncertainty shocks, and (ii) some parameters follow a Markov-switching process. Our Markov-switching procedure relies on the original constant-parameters model [Dynare code](https://www.aeaweb.org/articles?id=10.1257/aer.104.1.27). 

The MS model is linearized using a standard linearization procedure: each regime of the MS model is linearized around the ergodic steady state equilibrium through the Dynare software. We retrieve linearized equations from the analytical derivatives of the Jacobian matrix and then solve the model using the Cho (2016)'s forward method to solve the MS-DSGE model.

Note that the Jacobian matrix was created by typing 'Dynare cmr.mod' in the "PublicCodes_LT21/msdsge/dynare" folder.

## Library

Folder "library" contains all other Matlab functions. Folders "Bianchi_methods" and "Bianchi_toolbox" come from [Francesco Bianchi](https://sites.google.com/view/francescobianchi/home)'s library.
