% =========================================================================
% CODE FOR "Methods for Measuring Expectations and Uncertainty in
% Markov-Switching Models"
% The Journal of Econometrics, 2016, 190(1), pp. 7999" 
% Authors: Francesco Bianchi 
% =========================================================================
function [ me ] = cond_firstmoments7_pr( OMEGA, nreg, Thor, y, pr )
% This function computes the law of motion for first moments 
% of a MS-DSGE WITHOUT A MS-CONSTANT 
% assuming that the economy starts from the vector y and prob pr
% If y and pr are not provided, the code assumes that the economy is in
% steady state and compute the first moments for each of the regimes.
% ----------------------------------------------------------------------- %
% All results for first moments are in deviations from steady state, so the
% code can be used to compute impulse responses as long as the mean is
% regime-invariant. To do this, the value for y should be R*shocks in a
% model of the form S{t}=T*S{t-1}+R*eps{t}



n=(size(OMEGA,1)/nreg);
if nargin>3
    yt=y';
    ss=size(y,1);
else
    ss=1;
    y=zeros(1,n);
end
    
if nargin>4
    all=0;
else
    all=1;
end



%------------------------------------------------------------------------%
% First moments
Un=ones(1,n);

OMEGA_t(:,:,1)=eye(n*nreg);
for tt=2:Thor+1
    OMEGA_t(:,:,tt)=OMEGA*OMEGA_t(:,:,tt-1);
end
selq=repmat(eye(n),1,nreg);
me=zeros(n,ss,Thor+1,all*(nreg-1)+1);


fmyall=repmat(y,1,nreg)';
if all==1 % Compute conditional moments conditional on all regimes
          % This is what cond_moments3 does
    for ireg=1:nreg
        prmo=zeros(ss,nreg);
        prmo(:,ireg)=1;  
        prpr=kron(prmo,Un)';

        q0=fmyall.*prpr;
        for th=1:Thor+1
            me(:,:,th,ireg)=selq*OMEGA_t(:,:,th)*q0;
        end
%             qT(:,:,ireg)=qT_temp;
    end
else
    prmo=pr;
    prpr=kron(prmo,Un)';

    q0=fmyall.*prpr;
    for th=1:Thor+1
%             qT(:,:,th)=OMEGA_t(:,:,th)*q0;
        me(:,:,th)=selq*OMEGA_t(:,:,th)*q0;
    end

end

