function [ miu1, miu2, covmbar, sdbar ] = stst_ms_cf_rev( AUX, co )
% =========================================================================
% CODE FOR "Methods for Measuring Expectations and Uncertainty in
% Markov-Switching Models"
% The Journal of Econometrics, 2016, 190(1), pp. 7999" 
% Authors: Francesco Bianchi 
% =========================================================================
% Compute steady states using methods described in "Methods for..."
% miu1    -> First moments
% miu2    -> Second moments 
% covmbar -> Covariance matrix
% sdbar   -> Standard deviation
% corbar  -> Std/corr matrix


nout=nargout;
if nout>1
    [ OMEGA,M,HC,CSI,VV,LOMK,cc,DAC ] = from_AUX_to_MAT( AUX, co );
    n2m=size(CSI,1);
else
    [ OMEGA,M,HC ] = from_AUX_to_MAT( AUX, co );   
end

nm=size(OMEGA,1);m=size(HC,2);n=nm/m;

selQ=repmat(eye(n^2),1,m);
selq=repmat(eye(n),1,m);

% Compute ergodic probabilities
pibar=ss_prob(HC);

qbar=( (eye(nm)-OMEGA)\  (M * pibar) );
miu1=selq*qbar;

if nout>1
    Kbar=kron(pibar,reshape(eye(n),n*n,1));
%     keyboard
%     Qbar=( (eye(n2m)-CSI)\  (VV*Kbar+cc*pibar+DAC*qbar) );
    % 12/13/2015
    Qbar=( (eye(n2m)-CSI)\  ((VV+cc)*pibar+DAC*qbar) );
    miu2=selQ*Qbar;
    if nout>2
        covmbar=reshape(miu2,n,n)-miu1*miu1';
        if nout>3
           
            sdbar=diag(covmbar).^.5;
        end
    end
end

end

