% =========================================================================
% CODE FOR "Methods for Measuring Expectations and Uncertainty in
% Markov-Switching Models"
% The Journal of Econometrics, 2016, 190(1), pp. 7999" 
% Authors: Francesco Bianchi 
% =========================================================================
function [OME0s,OME0all,OME0str,sdstr,sdall]=unc_var_from_MAT4(MAT,HB,HC,method,control)


% method=2: Law of motion for second moments using sqrtm
% method=3: Law of motion for second moments using R and Q


[ TT,QQ,RR,HH,~,ZZ ] = from_MAT_to_T(MAT);
nsb = size(TT,3);
nsc = size(QQ,3);


% Compute covariance matrix MS states
for ib=1:nsb
    for jc=1:nsc
        T(:,:,(ib-1)*nsc+jc) = TT(:,:,ib);
        RQR=RR(:,:,ib)*QQ(:,:,jc)*RR(:,:,ib)';
        if method==2
        RQR05(:,:,(ib-1)*nsc+jc)=real(sqrtm((RQR+RQR')/2));
        elseif method==3
        RQ05(:,:,(ib-1)*nsc+jc)=RR(:,:,ib)*(QQ(:,:,jc)^0.5);
        end
    end
end

PP=kron(HB,HC);
if method==2
    OME0s = unc_var_ms_dsge3( T, RQR05, PP, 0 );
elseif method==3
    OME0s = unc_var_ms_dsge4_RQ( T, RQ05, PP, 0, control );
end
%  keyboard
OME0all=ZZ*OME0s*ZZ'+HH;
OME0str=ZZ*OME0s*ZZ';
sdstr=diag(OME0str).^0.5;
sdall=diag(OME0all).^0.5;
        
        