% =========================================================================
% CODE FOR "Methods for Measuring Expectations and Uncertainty in
% Markov-Switching Models"
% The Journal of Econometrics, 2016, 190(1), pp. 7999" 
% Authors: Francesco Bianchi 
% =========================================================================
function [ var, check, tcondvar, pbar, Qbar ] = unc_var_ms_dsge4_RQ( b, RQ, PP, T, control )
% This code computes the unconditional variance of a MS-DSGE model
% The code assumes that there is NO CONSTANT
% S_{t} = b*S_{t-1}+R*Q*eps_{t}
% b: Autoregressive matrix for dsge state vector
% RQ: Product of R and Q across different regime combinations
% In this case, the unconditional variance is equal to the second moment



n  = size(b,2);
n2 = n^2;
m  = size(PP,2);
k = size(RQ,2);
aux=1;
selQ=repmat(eye(n2),1,m);

% Steady state probabilities
pbar  = ss_prob( PP )';
% Law of motion
[AUX,~,CSIT] = mss_lom_nonhomog6_note_plus_dsge_RQ6_rev(b,RQ,PP,aux);

CSI = AUX{8};
VV  = AUX{9};

if control==1
    check=max(abs(eig(CSI)))<1;
else
    check=1;
end


if check==1
    Qbar=(eye(size(CSI))-CSI)\VV*pbar';
    thsecm=selQ*Qbar;
    thsecm=reshape(thsecm(1:n2),n,n);
    var = (thsecm+thsecm')/2;



    if T>0 % Compute evolution of variance up to T periods ahead conditional 
           % on being in regime j today for j=1...m
        QtildaT=zeros((n2+1)*m,T,m);
        tcondvar=zeros(n2,T,m);
        y0=zeros(1,n);
        co=0;

        for reg=1:m
            pt=zeros(1,m);
            pt(reg)=1;

            [ ~, Qtilda0 ] = construct_q0_Q0_rev( y0, pt, co, k );
            QtildaT(:,1,reg)=Qtilda0;
            for it=2:T
                QtildaT(:,it,reg)=CSIT*QtildaT(:,it-1,reg);
            end
            tcondvar(:,:,reg)=selQ*QtildaT(1:m*n2,:,reg);
        end
    end

else
    var = []; 
    tcondvar = []; 
    pbar = []; 
    Qbar = [];
end
























 