% =========================================================================
% CODE FOR "Methods for Measuring Expectations and Uncertainty in
% Markov-Switching Models"
% The Journal of Econometrics, 2016, 190(1), pp. 7999" 
% Authors: Francesco Bianchi 
% =========================================================================
function [vd,vol,check]=vardec5_from_MAT6(MAT,method,control)


% method=1: Lyapunov equation
% method=2: Law of motion for second moments using sqrtm
% method=3: Law of motion for second moments using R and Q


[ TT,QQ,RR,HH,~,ZZ ] = from_MAT_to_T(MAT);
nsb = size(TT,3);
nsc = size(QQ,3);
nsh = size(QQ,2);
ny  = size(ZZ,1);
vd  = zeros(ny,nsh+2,nsb*nsc);
vol = zeros(ny,nsb*nsc);
% Compute covariance matrix MS states
for ib=1:nsb
    T=TT(:,:,ib);
    R=RR(:,:,ib);

    for jc=1:nsc
        Q=QQ(:,:,jc);
        % 1.a) All shocks
        if method==1
            RQR=R*Q*R';
            RQR=(RQR+RQR')/2;
            OME0s=real(dlyap(T,RQR));
            check=1;
        elseif method==2
            RQR=R*Q*R';
            RQR=(RQR+RQR')/2;
            RQR05=real(RQR^0.5);
            [OME0s,check] = unc_var_ms_dsge3( T, RQR05, 1, 0, control );
        elseif method==3
            RQ=R*(Q^0.5);
            [OME0s,check] = unc_var_ms_dsge4_RQ( T, RQ, 1, 0, control );
%             keyboard
        end
        if check==1
            OME0all=ZZ*OME0s*ZZ'+HH;
            % 1.b) Only str shocks and no observ shock
            OME0str=ZZ*OME0s*ZZ';


            for sh = 1:nsh
                Qr=zeros(size(Q));Qr(sh,sh)=Q(sh,sh);

                % Only sh shock and no observ shock

                if method==1
                    RQrR=R*Qr*R';
                    RQrR=(RQrR+RQrR')/2;
                    OME0s=real(dlyap(T,RQrR));
                elseif method==2
                    RQrR=R*Qr*R';
                    RQrR=(RQrR+RQrR')/2;
                    RQrR05=real(RQrR^0.5);
                    OME0s = unc_var_ms_dsge3( T, RQrR05, 1, 0, 0 );
                elseif method==3
                    RQr=R*(Qr^0.5);
                    OME0s = unc_var_ms_dsge4_RQ( T, RQr, 1, 0, 0 );
                end
                OME0y=ZZ*OME0s*ZZ';
                % Decomp of total variance
                vd(:,sh,(jc-1)*nsb+ib,2)=diag(OME0y)./diag(OME0all);
                % Decomp of model implied variance
                vd(:,sh,(jc-1)*nsb+ib,1)=diag(OME0y)./diag(OME0str);
%                 disp('order for vd has been changed')
            end 

            % Contribution of observation errors 
            vd(:,nsh+1,(jc-1)*nsb+ib,1)=diag(HH)./diag(OME0all);
            vd(:,nsh+2,(jc-1)*nsb+ib,1)=diag(OME0str)./diag(OME0all);
            vd(:,nsh+1,(jc-1)*nsb+ib,2)=diag(HH)./diag(OME0all);
            vd(:,nsh+2,(jc-1)*nsb+ib,2)=diag(OME0str)./diag(OME0all);
            % "Model volatility" and overall volatility (including obs err)
            vol(:,(jc-1)*nsb+ib,1)=(diag(OME0str).^0.5);
            vol(:,(jc-1)*nsb+ib,2)=(diag(OME0all).^0.5);
            if min(isreal(vol))==0;keyboard;end

            
        end
    end  
end