% =========================================================================
% CODE by: Francesco Bianchi, Duke University
% =========================================================================
function [ssirfall_y,ssirfall_s] = dsge_imprf3( TT,QQ,RR,ZZ,nirf,signsh,varargin )


nsh  = size(QQ,1); % number of shocks
nvar = size(ZZ,1); % number of observed variables
nstv = size(TT,1); % number of DSGE states
nsb  = size(TT,3); % number of regimes for structural parameters
nsc  = size(QQ,3); % number of regimes for stochastic volatilities

noutput=nargout;ninput =nargin ;
ssirfall_y=zeros(nirf,nvar,nsh,nsb,nsc);
if noutput>1
    ssirfall_s=zeros(nirf,nstv,nsh,nsb,nsc);
end
if ninput<6
    signsh=ones(nsh,1);
end
if ninput>6
    names=varargin{1};
    vnames=names{1};
    if size(names,2)>1;shnames=names{2};else shnames=[];end
    if size(names,2)>2;snames=names{3};else snames=[];end
    if size(names,2)>3;regnames=names{4};else regnames=[];end
end
Zt=ZZ(:,:,1)';
for regb=1:nsb
    T=TT(:,:,regb);
    R=RR(:,:,regb);
    for regc=1:nsc
        Q05=chol(QQ(:,:,regc));
        shoi = 1;
        while shoi <= nsh
            impact = signsh(shoi)*Q05(:,shoi);
            ssirf  = zeros( nirf, nstv );
            % Compute IRF for each orthogonalized shock
            ssirf(1,:) = (R*impact)';
            for t=2:nirf
              ssirf(t,:) = (T*ssirf(t-1,:)')';
            end
            % Store results
            ssirfall_y(:,:,shoi,regb,regc) = ssirf * Zt;
            if noutput>1
                ssirfall_s(:,:,shoi,regb,regc) = ssirf;
            end
            shoi = shoi+1;
        end
    end
end
% keyboard
if ninput>6
    if nsb<5
        for isc=1:nsc
            figname=['Volatility regime ',num2str(isc)];
            [ tt ] = dsge_imprf_plot( ssirfall_y(:,:,:,:,1),...
                vnames,shnames,regnames,figname );
        end
    end
end


end

