function logMDD = fn_mdd_mhm(posthat,paramdraws,postdraws)
% 
%    Computing Marginal Data Density (or Marginal Likelihood)
%    The Modified Harmonic Means 
%    Geweke (1998, Econometric Reviews)
%
% --------------------
% posthat : posterior mode
% paramdraws : draws from posterior density
% postdraws : posterior draws
% --------------------
% logMDD: log of marginal data density (or marginal likelihood)
%
% by Stephane Lhuissier and Rita Zabelina
% 2013


%=================================================
%  (1) Compute posterior moments of the parameters
%=================================================

thetamean = mean(paramdraws,1)';
thetasig = cov(paramdraws,1);

todraws = size(paramdraws,1);
k = size(paramdraws,2);

p = (.1:.1:(9-1)*.1)';
pcrit = chi2inv(p,k);
    
[u,s,v]     = svd(thetasig);
md          = min(size(s)); 
bigev       = find(diag(s(1:md,1:md))>1e-6);
thetasigdim = length(bigev);
	
thetasiglndet = 0;

	for i = 1:k
      
		if i > thetasigdim
        s(i,i) = 0;
        else
        thetasiglndet = thetasiglndet+log(s(i,i));             
        s(i,i)    = 1/s(i,i);
        end
	end
	
thetasiginv  = u*s*u';

% record the minimum and maximum values of k(theta|Y)
k_min = min(postdraws(:,1));
k_max = max(postdraws(:,1));

%=================================================
%   (2) Compute density estimates
%=================================================
ndraws     = 0;
suminvlike = zeros(length(p),1);  

for j = 1:todraws

  thetasim = paramdraws(j,:)';
  
  post = postdraws(j,1);
  
  lnftheta = -log(p)-.5*k*log(2*pi)-.5*thetasiglndet-.5*(thetasim-thetamean)'*thetasiginv*(thetasim-thetamean);

  indtheta = ( (thetasim-thetamean)'*thetasiginv*(thetasim-thetamean) < pcrit );

  invlike = exp(lnftheta-post+posthat).*indtheta;

  if ~isinf(invlike) & ~isnan(invlike)

   suminvlike = suminvlike+invlike;
   ndraws = ndraws+1;

  end

end

%=================================================
%   (3) Estimate MDD
%=================================================
meaninvlike = suminvlike/ndraws;

logMDD = posthat-log(meaninvlike);



