function [lnlikelihood, imfhat, imfhat_select] = compute_likelihood(para,M_,oo_,options_,idx_fig)

% main features
nr           = 2;
nex          = M_.exo_nbr;
nendo        = M_.endo_nbr;
nvar         = sum(options_.ms.select_vec,1);

% asign values to parameters
p11          = para(1);
p22          = para(2);
kap_p        = para(3);
xip_p        = [para(4); para(5)];
xiw_p        = [para(6); para(7)];
iota_p       = [para(8); para(9)];
iotaw_p      = [para(10); para(11)];
b_p          = [para(12); para(13)];
Fomegabar_p  = [para(14); para(15)];
sigmaa_p     = [para(16); para(17)];
aptil_p      = [para(18); para(19)];
adytil_p     = [para(20); para(21)];
Sdoupr_p     = [para(22); para(23)];
mu_p         = [para(24); para(25)];
rhosigma_p   = [para(26); para(27)];                                    
stdsigma1_p  = [para(28); para(29)];

% Transition probabilities
P = [p11  1-p22; 1-p11  p22];

% ergodic value of some structural parameters
probss    = ss_prob(P);
mu_erg    = probss'*mu_p;
xip_erg   = probss'*xip_p;
xiw_erg   = probss'*xiw_p;
iota_erg  = probss'*iota_p;
iotaw_erg = probss'*iotaw_p;

% assign mu to M_ in order to compute steady state
M_ = set_myparam_value(M_,'mu_p',mu_erg);
M_ = set_myparam_value(M_,'xip_p',xip_erg);
M_ = set_myparam_value(M_,'xiw_p',xiw_erg);
M_ = set_myparam_value(M_,'iota_p',iota_erg);
M_ = set_myparam_value(M_,'iotaw_p',iotaw_erg);

% ergodic steady state (only some parameters affect steady state)
try
    oo_.steady_state = steady_(M_,options_,oo_);
catch
    lnlikelihood  = -1E13;
    imfhat        = NaN;
    imfhat_select = NaN;
    return;
end

if isempty(oo_.steady_state) % fail to compute steady state
    lnlikelihood  = -1E13;
    imfhat        = NaN;
    imfhat_select = NaN;
    return;
end

% memory allocation for MS-DSGE's matrices in the Cho's form
A = cell(2,1); B = cell(2,1); C = cell(2,1);

% assign values [cosntant parameters]
M_ = set_myparam_value(M_,'kap_p',kap_p);
    
for iS=1:2
    
    % assign values [MS parameters]
    M_ = set_myparam_value(M_,'xip_p',xip_p(iS));
    M_ = set_myparam_value(M_,'xiw_p',xiw_p(iS));
    M_ = set_myparam_value(M_,'iota_p',iota_p(iS));
    M_ = set_myparam_value(M_,'iotaw_p',iotaw_p(iS));
    M_ = set_myparam_value(M_,'b_p',b_p(iS));
    M_ = set_myparam_value(M_,'Fomegabar_p',Fomegabar_p(iS));
    M_ = set_myparam_value(M_,'sigmaa_p',sigmaa_p(iS));
    M_ = set_myparam_value(M_,'aptil_p',aptil_p(iS));
    M_ = set_myparam_value(M_,'adytil_p',adytil_p(iS));
    M_ = set_myparam_value(M_,'Sdoupr_p',Sdoupr_p(iS));
    M_ = set_myparam_value(M_,'mu_p',mu_p(iS));
    M_ = set_myparam_value(M_,'rhosigma_p',rhosigma_p(iS));
    M_ = set_myparam_value(M_,'stdsigma1_p',stdsigma1_p(iS));
            
    % extracts the A, B, C, and D matrices from the system's
    % Jacobian, where E_t\{ A*s_{t+1} + B*s_t + C*s_{t-1} + D*\epsilon_t \} = 0
    [a1,b1,b2,c1] = make_ABCD(M_,oo_);
    
    % model in Cho form 
    A{iS} = (-b1)\a1;
    B{iS} = (-b1)\b2;
    C{iS} = (-b1)\c1;
    
end

[OmegaK,GammaK,idx_control]=fmmsre_sl(P',A,B,C,[]);

if idx_control == 1  
    
    % Uncertainty shocks from MS-DSGE model
    imfhat = zeros(options_.ms.horizon+1,nendo,2);
    for iS=1:nr
        shock                                       = zeros(options_.ms.horizon+1,nex); 
        shock(1,string(M_.exo_names)=='e_sigma   ') = stdsigma1_p(iS);
        imfhat(:,:,iS)                              = simsirf(OmegaK{iS},GammaK{iS},shock,options_.ms.horizon+1); 
    end

    % Choice of endogenous variables 
    imfhat_select             = zeros(options_.ms.horizon+1,nvar,nr);
    for iS=1:nr
        imfhat_select(:,1,iS) = 100*cumsum(imfhat(:,string(M_.endo_names)=='gdp_obs        ',iS));
        imfhat_select(:,2,iS) = 100*imfhat(:,string(M_.endo_names)=='vix_obs        ',iS);
        imfhat_select(:,3,iS) = 400*imfhat(:,string(M_.endo_names)=='premium_obs    ',iS);
    end
    imfhat_select             = imfhat_select(:,find(options_.ms.select_vec==1)',:);

    % Display Figure, 1: YES, 0: NO
    if (idx_fig)
        plot_IRFs_DSGE_VAR;
        pause(0.00000000001)
    end

    psitheta = imfhat_select(:);
    psitheta = psitheta(options_.ms.indx);

    % Compute the log-likehood
    criterion      = (options_.ms.psihat-psitheta)'*options_.ms.inv_Vhat*(options_.ms.psihat-psitheta);
    lnlikelihood   = (size(options_.ms.psihat,1)/2*log(1/2/pi)-1/2*options_.ms.logdetVhat-1/2*criterion);

    if isnan(lnlikelihood)
        lnlikelihood = -1E13;
    end
else
	lnlikelihood = -1E13;
end
