%
% Generate Figure 5
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath(genpath('../../../'))

% recover output from Dynare 
load('cmr_results','M_','oo_','options_');

% load MS-DSGE results
load('../results/baseline/posterior_mode','Postmode');

% IRFs from estimated MS-VAR model
run('../get_MS_VAR_resp.m');

% evaluation of the log-likelihood at the initial values
[~,~,imfhat_select] = compute_likelihood(Postmode, M_, oo_, options_, 0);

t = 0:options_.ms.horizon;

make_it_tight = true;                     % GAP        LOWER UPPER  LEFT RIGHT
subplot = @(m,n,p) subtightplot (m, n, p, [0.15 0.07], [0.09 0.06], [0.08 0.02]);
if ~make_it_tight,  clear subplot;  end

figure
set(gcf,'Position',[100 100 500 500])
subplot(2,3,1)
plot(t,imfhat_select(:,1,1),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,3,1),'-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,1,1),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,5,1),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,2,1),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,4,1),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,zeros(options_.ms.horizon+1,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
ylim([min([irf_quan_68_90(1:options_.ms.horizon+1,1,1,1); irf_quan_68_90(1:options_.ms.horizon+1,1,5,1); irf_quan_68_90(1:options_.ms.horizon+1,1,1,2); irf_quan_68_90(1:options_.ms.horizon+1,1,5,2)]) ...
      max([irf_quan_68_90(1:options_.ms.horizon+1,1,1,1); irf_quan_68_90(1:options_.ms.horizon+1,1,5,1); irf_quan_68_90(1:options_.ms.horizon+1,1,1,2); irf_quan_68_90(1:options_.ms.horizon+1,1,5,2)])])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel A. Output [1]');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percent')
%ylabel({'Percent Deviation';'from Steady State'})
xlabel('quarters after shock')
subplot(2,3,4)
plot(t,imfhat_select(:,1,2),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,3,2),'-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,1,2),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,5,2),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,2,2),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,1,4,2),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,zeros(options_.ms.horizon+1,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
ylim([min([irf_quan_68_90(1:options_.ms.horizon+1,1,1,1); irf_quan_68_90(1:options_.ms.horizon+1,1,5,1); irf_quan_68_90(1:options_.ms.horizon+1,1,1,2); irf_quan_68_90(1:options_.ms.horizon+1,1,5,2)]) ...
      max([irf_quan_68_90(1:options_.ms.horizon+1,1,1,1); irf_quan_68_90(1:options_.ms.horizon+1,1,5,1); irf_quan_68_90(1:options_.ms.horizon+1,1,1,2); irf_quan_68_90(1:options_.ms.horizon+1,1,5,2)])])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel B. Output [2]');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percent')
%ylabel({'Percent Deviation';'from Steady State'})
xlabel('quarters after shock')
subplot(2,3,2)
plot(t,imfhat_select(:,2,1),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,3,1),'-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,1,1),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,5,1),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,2,1),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,4,1),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,zeros(options_.ms.horizon+1,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
ylim([min([irf_quan_68_90(1:options_.ms.horizon+1,2,1,1); irf_quan_68_90(1:options_.ms.horizon+1,2,5,1); irf_quan_68_90(1:options_.ms.horizon+1,2,1,2); irf_quan_68_90(1:options_.ms.horizon+1,2,5,2)]) ...
      max([irf_quan_68_90(1:options_.ms.horizon+1,2,1,1); irf_quan_68_90(1:options_.ms.horizon+1,2,5,1); irf_quan_68_90(1:options_.ms.horizon+1,2,1,2); irf_quan_68_90(1:options_.ms.horizon+1,2,5,2)])])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel C. VIX [1]');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel({'Percentage Points'})
xlabel('quarters after shock')
subplot(2,3,5)
plot(t,imfhat_select(:,2,2),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,3,2),'-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,1,2),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,5,2),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,2,2),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,2,4,2),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,zeros(options_.ms.horizon+1,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
ylim([min([irf_quan_68_90(1:options_.ms.horizon+1,2,1,1); irf_quan_68_90(1:options_.ms.horizon+1,2,5,1); irf_quan_68_90(1:options_.ms.horizon+1,2,1,2); irf_quan_68_90(1:options_.ms.horizon+1,2,5,2)]) ...
      max([irf_quan_68_90(1:options_.ms.horizon+1,2,1,1); irf_quan_68_90(1:options_.ms.horizon+1,2,5,1); irf_quan_68_90(1:options_.ms.horizon+1,2,1,2); irf_quan_68_90(1:options_.ms.horizon+1,2,5,2)])])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel D. VIX [2]');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel({'Percentage Points'})
xlabel('quarters after shock')
subplot(2,3,3)
plot(t,imfhat_select(:,3,1),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,3,1),'-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,1,1),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,5,1),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,2,1),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,4,1),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,zeros(options_.ms.horizon+1,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
ylim([min([irf_quan_68_90(1:options_.ms.horizon+1,3,1,1); irf_quan_68_90(1:options_.ms.horizon+1,3,5,1); irf_quan_68_90(1:options_.ms.horizon+1,3,1,2); irf_quan_68_90(1:options_.ms.horizon+1,3,5,2)]) ...
      max([irf_quan_68_90(1:options_.ms.horizon+1,3,1,1); irf_quan_68_90(1:options_.ms.horizon+1,3,5,1); irf_quan_68_90(1:options_.ms.horizon+1,3,1,2); irf_quan_68_90(1:options_.ms.horizon+1,3,5,2)])])
xlim([0 16])
set(gca,'xgrid','on')
set(gca,'ygrid','on')
tt=title('Panel E. Credit spread [1]');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percentage Points')
xlabel('quarters after shock')
subplot(2,3,6)
aa=plot(t,imfhat_select(:,3,2),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
hold on
bb=plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,3,2),'-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
cc=plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,1,2),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,5,2),':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
dd=plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,2,2),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_quan_68_90(1:options_.ms.horizon+1,3,4,2),'-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,zeros(options_.ms.horizon+1,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
ylim([min([irf_quan_68_90(1:options_.ms.horizon+1,3,1,1); irf_quan_68_90(1:options_.ms.horizon+1,3,5,1); irf_quan_68_90(1:options_.ms.horizon+1,3,1,2); irf_quan_68_90(1:options_.ms.horizon+1,3,5,2)]) ...
      max([irf_quan_68_90(1:options_.ms.horizon+1,3,1,1); irf_quan_68_90(1:options_.ms.horizon+1,3,5,1); irf_quan_68_90(1:options_.ms.horizon+1,3,1,2); irf_quan_68_90(1:options_.ms.horizon+1,3,5,2)])])
xlim([0 16])
set(gca,'xgrid','on')
set(gca,'ygrid','on')
tt=title('Panel F. Credit spread [2]');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percentage Points')
xlabel('quarters after shock')
lgd=legend([aa,bb,dd,cc],'Mode (MS-DSGE)','Median (MS-SVAR)','68% error bands (MS-SVAR)','90% error bands (MS-SVAR)','Location','NorthEast');
lgd.FontSize = 7;

set(gcf,'PaperUnits','inches');
set(gcf,'PaperSize',[8 6]);
set(gcf,'PaperPositionMode','manual');
set(gcf,'PaperPosition',[0 0 8 6]);

set(gcf,'renderer','painters');
eval(['print -dpdf Fig5']);
