%
% Generate Figure 6
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath(genpath('../../../'))

% =========================================================================
% STEP 0. LOAD DATA AND PARAMETERS 
% =========================================================================

% recover output from Dynare 
load('cmr_results','M_','oo_','options_');

% recover results of posterior mode
load('../results/baseline/posterior_mode.mat','xh','Postmode');

% main features
nex           = M_.exo_nbr;
nendo         = M_.endo_nbr;
npara         = M_.param_nbr;
imstp         = 16+1;
nreg          = 2;
Opt.IRvs      = 1;
Opt.IRT       = imstp;
Opt.InfoAdj   = 0;

% asign values to parameters
% Sdoupr_p      = xh(4:5);% [0.5001; 1.3017];
% mu_p          = xh(6:7);%[0.2011; 0.0536];
% rhosigma_p    = xh(8:9);%[0.5953; 0.7504];
% stdsigma1_p   = xh(10:11);%[0.4111; 0.4587];
% kap_p         = xh(3);%0.8389;
p11          = Postmode(1);
p22          = Postmode(2);
kap_p        = Postmode(3);
xip_p        = [Postmode(4); Postmode(5)];
xiw_p        = [Postmode(6); Postmode(7)];
iota_p       = [Postmode(8); Postmode(9)];
iotaw_p      = [Postmode(10); Postmode(11)];
b_p          = [Postmode(12); Postmode(13)];
Fomegabar_p  = [Postmode(14); Postmode(15)];
sigmaa_p     = [Postmode(16); Postmode(17)];
aptil_p      = [Postmode(18); Postmode(19)];
adytil_p     = [Postmode(20); Postmode(21)];
Sdoupr_p     = [Postmode(22); Postmode(22)];%[Postmode(22); Postmode(23)];
mu_p         = [Postmode(24); Postmode(25)];%[Postmode(24); Postmode(25)];
rhosigma_p   = [Postmode(26); Postmode(26)];%[Postmode(26); Postmode(27)];                                    
stdsigma1_p  = [Postmode(28); Postmode(28)];%[Postmode(28); Postmode(29)];

% Transition probabilities
%p11  = xh(1);
%p22  = xh(2);
P    = [p11  1-p22; 1-p11  p22];

% ergodic value of mu
probss  = ss_prob(P);
mu_erg  = probss'*mu_p;

% memory allocation for MS-DSGE's matrices in the Cho's form
A = cell(nreg,1); B = cell(nreg,1); C = cell(nreg,1);

% assign mu to M_ in order to compute steady state
M_ = set_myparam_value(M_,'mu_p',mu_erg);

% ergodic steady state
oo_.steady_state = steady_(M_,options_,oo_); 

% =========================================================================
% STEP 1. Uncertainty shock from MS-DSGE model [at the mode]
% =========================================================================
    M_ = set_myparam_value(M_,'kap_p',kap_p);
for iS=1:nreg
    M_ = set_myparam_value(M_,'xip_p',xip_p(iS));
    M_ = set_myparam_value(M_,'xiw_p',xiw_p(iS));
    M_ = set_myparam_value(M_,'iota_p',iota_p(iS));
    M_ = set_myparam_value(M_,'iotaw_p',iotaw_p(iS));
    M_ = set_myparam_value(M_,'aptil_p',aptil_p(iS));
    M_ = set_myparam_value(M_,'adytil_p',adytil_p(iS));
    M_ = set_myparam_value(M_,'mu_p',mu_p(iS));
    M_ = set_myparam_value(M_,'Sdoupr_p',Sdoupr_p(iS));
    M_ = set_myparam_value(M_,'rhosigma_p',rhosigma_p(iS));
    M_ = set_myparam_value(M_,'stdsigma1_p',stdsigma1_p(iS));
    
    % extracts the A, B, C, and D matrices from the system's
    % Jacobian, where E_t\{ A*s_{t+1} + B*s_t + C*s_{t-1} + D*\epsilon_t \} = 0
    [a1,b1,b2,c1] = make_ABCD(M_,oo_);
  
    % model in Cho form 
    A{iS} = (-b1)\a1;
    B{iS} = (-b1)\b2;
    C{iS} = (-b1)\c1;
end

[OmegaK,GammaK,idx_control]=fmmsre_sl(P',A,B,C,[]);

% Uncertainty shocks from MS-DSGE model
irf = zeros(imstp,nendo,nreg);
for iS=1:nreg
    shock                                       = zeros(imstp,nex);
    shock(1,string(M_.exo_names)=='e_sigma   ') = stdsigma1_p(1);
    irf(:,:,iS)                                 = simsirf(OmegaK{iS},GammaK{iS},shock,imstp);
end

% =========================================================================
% STEP 2. Uncertainty shock from MS-DSGE model [counterfactual p11]
% =========================================================================
idx_        = 0.10:0.1:1; % Range of probabilities
irfctf_p11  = zeros(imstp,nendo,nreg,length(idx_)); % memory allocation

for iR=1:length(idx_)
    
    % new prob.
    p11     = idx_(iR);
    P       = [p11  1-p22; 1-p11  p22];
    
%     % ergodic value of mu
%     probss  = ss_prob(P);
%     mu_erg  = probss'*mu_p;
%     
%     % assign mu to M_ in order to compute steady state
%     M_ = set_myparam_value(M_,'mu_p',mu_erg);
%     
%     % ergodic steady state
%     oo_.steady_state = steady_(M_,options_,oo_);
    
    % memory allocation for MS-DSGE's matrices in the Cho's form
    A = cell(nreg,1); B = cell(nreg,1); C = cell(nreg,1);
    
    for iS=1:nreg
        %
        M_ = set_myparam_value(M_,'xip_p',xip_p(iS));
        M_ = set_myparam_value(M_,'xiw_p',xiw_p(iS));
        M_ = set_myparam_value(M_,'iota_p',iota_p(iS));
        M_ = set_myparam_value(M_,'iotaw_p',iotaw_p(iS));
        M_ = set_myparam_value(M_,'aptil_p',aptil_p(iS));
        M_ = set_myparam_value(M_,'adytil_p',adytil_p(iS));
        M_ = set_myparam_value(M_,'mu_p',mu_p(iS));
        M_ = set_myparam_value(M_,'Sdoupr_p',Sdoupr_p(iS));
        M_ = set_myparam_value(M_,'rhosigma_p',rhosigma_p(iS));
        M_ = set_myparam_value(M_,'stdsigma1_p',stdsigma1_p(iS));
        
        % extracts the A, B, C, and D matrices from the system's
        % Jacobian, where E_t\{ A*s_{t+1} + B*s_t + C*s_{t-1} + D*\epsilon_t \} = 0
        [a1,b1,b2,c1] = make_ABCD(M_,oo_);
        
        % model in Cho form
        A{iS} = (-b1)\a1;
        B{iS} = (-b1)\b2;
        C{iS} = (-b1)\c1;
    end
    
    [OmegaCtf,GammaCtf,idx_control]=fmmsre_sl(P',A,B,C,[]);
    
    % Uncertainty shock from MS-DSGE model
    for iS=1:nreg
        shock                                       = zeros(imstp,nex);
        shock(1,string(M_.exo_names)=='e_sigma   ') = stdsigma1_p(1);
        irfctf_p11(:,:,iS,iR)                       = simsirf(OmegaCtf{iS},GammaCtf{iS},shock,imstp);
    end
    
end

% =========================================================================
% STEP 2. Uncertainty shock from MS-DSGE model [counterfactual p22]
% =========================================================================
irfctf_p22  = zeros(imstp,nendo,nreg,length(idx_)); % memory allocation
p11         = Postmode(1);

for iR=1:length(idx_)
    
    % new prob.
    p22     = idx_(iR);
    P       = [p11  1-p22; 1-p11  p22];

%     % ergodic value of mu
%     probss  = ss_prob(P);
%     mu_erg  = probss'*mu_p;
%     
%     % assign mu to M_ in order to compute steady state
%     M_ = set_myparam_value(M_,'mu_p',mu_erg);
%     
%     % ergodic steady state
%     oo_.steady_state = steady_(M_,options_,oo_);
    
    % memory allocation for MS-DSGE's matrices in the Cho's form
    A = cell(nreg,1); B = cell(nreg,1); C = cell(nreg,1);
    
    for iS=1:nreg
        %
        M_ = set_myparam_value(M_,'xip_p',xip_p(iS));
        M_ = set_myparam_value(M_,'xiw_p',xiw_p(iS));
        M_ = set_myparam_value(M_,'iota_p',iota_p(iS));
        M_ = set_myparam_value(M_,'iotaw_p',iotaw_p(iS));
        M_ = set_myparam_value(M_,'aptil_p',aptil_p(iS));
        M_ = set_myparam_value(M_,'adytil_p',adytil_p(iS));
        M_ = set_myparam_value(M_,'mu_p',mu_p(iS));
        M_ = set_myparam_value(M_,'Sdoupr_p',Sdoupr_p(iS));
        M_ = set_myparam_value(M_,'rhosigma_p',rhosigma_p(iS));
        M_ = set_myparam_value(M_,'stdsigma1_p',stdsigma1_p(iS));
            
        % extracts the A, B, C, and D matrices from the system's
        % Jacobian, where E_t\{ A*s_{t+1} + B*s_t + C*s_{t-1} + D*\epsilon_t \} = 0
        [a1,b1,b2,c1] = make_ABCD(M_,oo_);

        % model in Cho form 
        A{iS} = (-b1)\a1;
        B{iS} = (-b1)\b2;
        C{iS} = (-b1)\c1;
    end

    [OmegaCtf,GammaCtf,idx_control]=fmmsre_sl(P',A,B,C,[]);
    
    % Uncertainty shock from MS-DSGE model
    for iS=1:nreg
        shock                                       = zeros(imstp,nex);
        shock(1,string(M_.exo_names)=='e_sigma   ') = stdsigma1_p(1);
        irfctf_p22(:,:,iS,iR)                       = simsirf(OmegaCtf{iS},GammaCtf{iS},shock,imstp);
    end
end

% =========================================================================
% STEP 4. Figure
% =========================================================================
make_it_tight = true;                     % GAP        LOWER UPPER  LEFT RIGHT
subplot = @(m,n,p) subtightplot (m, n, p, [0.10 0.13], [0.11 0.06], [0.09 0.01]);
if ~make_it_tight,  clear subplot;  end

% blue color [p11]
bmap = winter(length(idx_));

% red color [p22]
rmap = autumn(length(idx_));

t = 0:imstp-1;

make_it_tight = true;                     % GAP        LOWER UPPER  LEFT RIGHT
subplot = @(m,n,p) subtightplot (m, n, p, [0.12 0.12], [0.09 0.06], [0.08 0.02]);
if ~make_it_tight,  clear subplot;  end

figure
set(gcf,'Position',[100 100 500 500])
% subplot(2,3,1)
% for i=1:length(idx_)
%     aa(i)=plot(t,100*cumsum(irfctf_p11(:,string(M_.endo_names)=='gdp_obs        ',1,i)),'color',bmap(i,:),'LineWidth',1.0);
%     hold on
% end
% for i=1:length(idx_)
%     plot(t,100*cumsum(irfctf_p22(:,string(M_.endo_names)=='gdp_obs        ',1,i)),'color',rmap(i,:),'LineWidth',1.0)
%     hold on
% end
% cc=plot(t,100*cumsum(irf(:,string(M_.endo_names)=='gdp_obs        ',1)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
% hold on
% plot(t,zeros(imstp,1),':k','LineWidth',1.5)
% set(gca,'xtick',[0 2 4 8 16])
% xlim([0 16])
% set(gca,'xgrid','on') 
% set(gca,'ygrid','on')
% tt=title('Panel A. Output [1]');
% set(tt, 'horizontalAlignment', 'left')
% set(tt, 'units', 'normalized')
% h = get(tt, 'position');
% set(tt, 'position', [0 1 0])
% ylabel('Percent')
% %ylabel({'Percent Deviation';'from Steady State'})
% xlabel('quarters after shock')
% ylim([-2 0.1])

% subplot(2,2,4)
% for i=1:length(idx_)
%     plot(t,100*cumsum(irfctf_p11(:,string(M_.endo_names)=='gdp_obs        ',2,i)),'color',bmap(i,:),'LineWidth',1.0);
%     hold on
% end
% for i=1:length(idx_)
%     bb(i)=plot(t,100*cumsum(irfctf_p22(:,string(M_.endo_names)=='gdp_obs        ',2,i)),'color',rmap(i,:),'LineWidth',1.0);
%     hold on
% end
% cc=plot(t,100*cumsum(irf(:,string(M_.endo_names)=='gdp_obs        ',2)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
% hold on
% plot(t,zeros(imstp,1),':k','LineWidth',1.5)
% set(gca,'xtick',[0 2 4 8 16])
% xlim([0 16])
% set(gca,'xgrid','on') 
% set(gca,'ygrid','on')
% tt=title('Panel B. Output [2]');
% set(tt, 'horizontalAlignment', 'left')
% set(tt, 'units', 'normalized')
% h = get(tt, 'position');
% set(tt, 'position', [0 1 0])
% ylabel('Percent')
% %ylabel({'Percent Deviation';'from Steady State'})
% xlabel('quarters after shock')
% ylim([-2 0.1])


subplot(2,2,1)
for i=1:length(idx_)
    plot(t,400*(irfctf_p11(:,string(M_.endo_names)=='premium_obs    ',1,i)),'color',bmap(i,:),'LineWidth',1.0);
    hold on
end
for i=1:length(idx_)
    plot(t,400*(irfctf_p22(:,string(M_.endo_names)=='premium_obs    ',1,i)),'color',rmap(i,:),'LineWidth',1.0);
    hold on
end
cc=plot(t,400*(irf(:,string(M_.endo_names)=='premium_obs    ',1)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
hold on
plot(t,zeros(imstp,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel A. Credit spread [1] - level');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percentage Points')
%ylabel({'Percent Deviation';'from Steady State'})
xlabel('quarters after shock')
%ylim([-2 10])

subplot(2,2,2)
for i=1:length(idx_)
    aa(i)=plot(t,400*(irfctf_p11(:,string(M_.endo_names)=='premium_obs    ',1,i))-400*(irf(:,string(M_.endo_names)=='premium_obs    ',1)),'color',bmap(i,:),'LineWidth',1.0);
    hold on
end
for i=1:length(idx_)
    bb(i)=plot(t,400*(irfctf_p22(:,string(M_.endo_names)=='premium_obs    ',1,i))-400*(irf(:,string(M_.endo_names)=='premium_obs    ',1)),'color',rmap(i,:),'LineWidth',1.0);
    hold on
end
%cc=plot(t,400*(irf(:,string(M_.endo_names)=='premium_obs    ',1)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
hold on
plot(t,zeros(imstp,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel C. Credit spread [1] - deviation');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percentage Points')
%ylabel({'Percent Deviation';'from Steady State'})
xlabel('quarters after shock')
ylim([-0.08 0.02])
leg2=legend([cc,aa(1:10),bb(1:10)],'Mode','$p_{11}=0.10$','$p_{11}=0.20$','$p_{11}=0.30$','$p_{11}=0.40$','$p_{11}=0.50$',...
                '$p_{11}=0.60$','$p_{11}=0.70$','$p_{11}=0.80$','$p_{11}=0.90$','$p_{11}=1.00$', ...
                '$p_{22}=0.10$','$p_{22}=0.20$','$p_{22}=0.30$','$p_{22}=0.40$','$p_{22}=0.50$',...
                '$p_{22}=0.60$','$p_{22}=0.70$','$p_{22}=0.80$','$p_{22}=0.90$','$p_{22}=1.00$','Location','NorthEast');%,'Orientation','horizontal');
set(leg2,'FontSize',8);
set(leg2,'Interpreter','latex');
leg2.NumColumns=2;


subplot(2,2,3)
for i=1:length(idx_)
    plot(t,400*(irfctf_p11(:,string(M_.endo_names)=='premium_obs    ',2,i)),'color',bmap(i,:),'LineWidth',1.0);
    hold on
end
for i=1:length(idx_)
    bb(i)=plot(t,400*(irfctf_p22(:,string(M_.endo_names)=='premium_obs    ',2,i)),'color',rmap(i,:),'LineWidth',1.0);
    hold on
end
cc=plot(t,400*(irf(:,string(M_.endo_names)=='premium_obs    ',2)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
hold on
plot(t,zeros(imstp,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel B. Credit spread [2] - level');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percentage Points')
%ylabel({'Percent Deviation';'from Steady State'})
xlabel('quarters after shock')
%ylim([-2 10])


subplot(2,2,4)
for i=1:length(idx_)
    plot(t,400*(irfctf_p11(:,string(M_.endo_names)=='premium_obs    ',2,i))-400*(irf(:,string(M_.endo_names)=='premium_obs    ',2)),'color',bmap(i,:),'LineWidth',1.0);
    hold on
end
for i=1:length(idx_)
    bb(i)=plot(t,400*(irfctf_p22(:,string(M_.endo_names)=='premium_obs    ',2,i))-400*(irf(:,string(M_.endo_names)=='premium_obs    ',2)),'color',rmap(i,:),'LineWidth',1.0);
    hold on
end
%cc=plot(t,400*(irf(:,string(M_.endo_names)=='premium_obs    ',2)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k');
hold on
plot(t,zeros(imstp,1),':k','LineWidth',1.5)
set(gca,'xtick',[0 2 4 8 16])
xlim([0 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
tt=title('Panel D. Credit spread [2] - deviation');
set(tt, 'horizontalAlignment', 'left')
set(tt, 'units', 'normalized')
h = get(tt, 'position');
set(tt, 'position', [0 1 0])
ylabel('Percentage Points')
%ylabel({'Percent Deviation';'from Steady State'})
xlabel('quarters after shock')
ylim([-0.01 0.02])

set(gcf,'PaperUnits','inches');
set(gcf,'PaperSize',[7 6]);
set(gcf,'PaperPositionMode','manual');
set(gcf,'PaperPosition',[0 0 7 6]);

set(gcf,'renderer','painters');
eval(['print -dpdf Fig7']);


% %leg1=legend([cc,aa(1:10)],'Mode','$p_{11}=0.10$','$p_{11}=0.20$','$p_{11}=0.30$','$p_{11}=0.40$','$p_{11}=0.50$',...
% %                '$p_{11}=0.60$','$p_{11}=0.70$','$p_{11}=0.80$','$p_{11}=0.90$','$p_{11}=1.00$','Location','SouthEast');
% leg2=legend([cc,aa(1:10),bb(1:10)],'Mode','$p_{11}=0.10$','$p_{11}=0.20$','$p_{11}=0.30$','$p_{11}=0.40$','$p_{11}=0.50$',...
%                 '$p_{11}=0.60$','$p_{11}=0.70$','$p_{11}=0.80$','$p_{11}=0.90$','$p_{11}=1.00$', ...
%                 '$p_{22}=0.10$','$p_{22}=0.20$','$p_{22}=0.30$','$p_{22}=0.40$','$p_{22}=0.50$',...
%                 '$p_{22}=0.60$','$p_{22}=0.70$','$p_{22}=0.80$','$p_{22}=0.90$','$p_{22}=1.00$','Location','SouthEast');%,'Orientation','horizontal');
% set(leg2,'FontSize',8);
% set(leg2,'Interpreter','latex');
% leg2.NumColumns=2;
%
%
% subplot(2,3,2)
% for i=1:length(idx_)
%     plot(t,100*(irfctf_p11(:,string(M_.endo_names)=='vix_obs        ',1,i)),'color',bmap(i,:),'LineWidth',1.0)
%     hold on
% end
% for i=1:length(idx_)
%     plot(t,100*(irfctf_p22(:,string(M_.endo_names)=='vix_obs        ',1,i)),'color',rmap(i,:),'LineWidth',1.0)
%     hold on
% end
% plot(t,100*(irf(:,string(M_.endo_names)=='vix_obs        ',1)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
% hold on
% plot(t,zeros(imstp,1),':k','LineWidth',1.5)
% set(gca,'xtick',[0 2 4 8 16])
% xlim([0 16])
% set(gca,'xgrid','on') 
% set(gca,'ygrid','on')
% tt=title('Panel C. Uncertainty [1]');
% set(tt, 'horizontalAlignment', 'left')
% set(tt, 'units', 'normalized')
% h = get(tt, 'position');
% set(tt, 'position', [0 1 0])
% ylabel({'Percentage Points'})
% xlabel('quarters after shock')
% ylim([-2 10])
%
% subplot(2,3,5)
% for i=1:length(idx_)
%     plot(t,100*(irfctf_p11(:,string(M_.endo_names)=='vix_obs        ',2,i)),'color',bmap(i,:),'LineWidth',1.0)
%     hold on
% end
% for i=1:length(idx_)
%     plot(t,100*(irfctf_p22(:,string(M_.endo_names)=='vix_obs        ',2,i)),'color',rmap(i,:),'LineWidth',1.0)
%     hold on
% end
% plot(t,100*(irf(:,string(M_.endo_names)=='vix_obs        ',2)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
% hold on
% plot(t,zeros(imstp,1),':k','LineWidth',1.5)
% set(gca,'xtick',[0 2 4 8 16])
% xlim([0 16])
% set(gca,'xgrid','on') 
% set(gca,'ygrid','on')
% tt=title('Panel D. Uncertainty [2]');
% set(tt, 'horizontalAlignment', 'left')
% set(tt, 'units', 'normalized')
% h = get(tt, 'position');
% set(tt, 'position', [0 1 0])
% ylabel({'Percentage Points'})
% xlabel('quarters after shock')
% ylim([-2 10])
%
% subplot(2,3,3)
% for i=1:length(idx_)
%     plot(t,400*(irfctf_p11(:,string(M_.endo_names)=='premium_obs    ',1,i)),'color',bmap(i,:),'LineWidth',1.0)
%     hold on
% end
% for i=1:length(idx_)
%     plot(t,400*(irfctf_p22(:,string(M_.endo_names)=='premium_obs    ',1,i)),'color',rmap(i,:),'LineWidth',1.0)
%     hold on
% end
% plot(t,400*(irf(:,string(M_.endo_names)=='premium_obs    ',1)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
% hold on
% plot(t,zeros(imstp,1),':k','LineWidth',1.5)
% set(gca,'xtick',[0 2 4 8 16])
% xlim([0 16])
% set(gca,'xgrid','on') 
% set(gca,'ygrid','on')
% tt=title('Panel E. Premium [1]');
% set(tt, 'horizontalAlignment', 'left')
% set(tt, 'units', 'normalized')
% h = get(tt, 'position');
% set(tt, 'position', [0 1 0])
% ylabel({'Percentage Points'})
% xlabel('quarters after shock')
% ylim([-0.05 0.8])
%
% subplot(2,3,6)
% for i=1:length(idx_)
%     plot(t,400*(irfctf_p11(:,string(M_.endo_names)=='premium_obs    ',2,i)),'color',bmap(i,:),'LineWidth',1.)
%     hold on
% end
% for i=1:length(idx_)
%     plot(t,400*(irfctf_p22(:,string(M_.endo_names)=='premium_obs    ',2,i)),'color',rmap(i,:),'LineWidth',1.)
%     hold on
% end
% plot(t,400*(irf(:,string(M_.endo_names)=='premium_obs    ',2)),'-s','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','k','color','k')
% hold on
% plot(t,zeros(imstp,1),':k','LineWidth',1.5)
% set(gca,'xtick',[0 2 4 8 16])
% xlim([0 16])
% set(gca,'xgrid','on') 
% set(gca,'ygrid','on')
% tt=title('Panel F. Premium [2]');
% set(tt, 'horizontalAlignment', 'left')
% set(tt, 'units', 'normalized')
% h = get(tt, 'position');
% set(tt, 'position', [0 1 0])
% ylabel({'Percentage Points'})
% xlabel('quarters after shock')
% ylim([-0.05 0.8])
% 
% set(gcf,'PaperUnits','inches');
% set(gcf,'PaperSize',[8 4]);
% set(gcf,'PaperPositionMode','manual');
% set(gcf,'PaperPosition',[0 0 8 4]);
% 
% set(gcf,'renderer','painters');
% eval(['print -dpdf Fig6bis']);
