%
% Generate Table 5
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath('../../../library')

name_file = 'baseline';

load(['../results/',name_file,'/posterior_draws_1.mat']);
load(['../results/',name_file,'/posterior_mode.mat']);

nuse    = 10;
nburn   = 0.10*size(parasimm,1);
parasim = forevery(parasimm(nburn+1:end,:),nuse);

Postdraw_med = median(parasim,1);
Postdraw_mea = mean(parasim,1);
Postdraw_lim = hpdint(parasim,0.90,0);

PostMatrix = [xh Postdraw_lim'];
%PostMatrix = [Postdraw_mea' Postdraw_lim'];

diary Tab4.tex
fprintf('\n')
fprintf('\\begin{table}\n')
fprintf('\\center\n')
fprintf('\\label{posterior_table}\n')
fprintf('\\begin{tabular}{llccccccccc}\n')
fprintf(' \\hline\n ')
fprintf(' \\hline\n')
fprintf(' & & \\multicolumn{3}{c}{Prior} & & \\multicolumn{3}{c}{Posterior} \\\\\n')
fprintf(' \\cline{3-5}\n')
fprintf(' \\cline{7-9}\n')
fprintf(' Coefficient & Description & Density & Mean & Std Dev & & Mode & [5; & 95] \\\\\n')
fprintf(' \\hline\n')
fprintf(' $p_{11}$ & Transition matrix  & D & 0.89 & 0.05 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(1,1), PostMatrix(1,2), PostMatrix(1,3))
fprintf(' $p_{22}$ & Transition matrix & D & 0.93 & 0.05 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(2,1), PostMatrix(2,2), PostMatrix(2,3))      
fprintf(' $\\kappa$ & Measurement VIX & N & 1.00 & 2.00 & & $%.4f$ & $%.4f$  & $%.4f$\\\\\n', ...
          PostMatrix(3,1), PostMatrix(3,2), PostMatrix(3,3))
fprintf(' $S''''(\\chi_t=1)$ & Investment adjustment costs  & G & 0.75 & 0.50 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(4,1), PostMatrix(4,2), PostMatrix(4,3))
fprintf(' $S''''(\\chi_t=2)$ & Investment adjustment costs  & G & 0.75 & 0.50 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(5,1), PostMatrix(5,2), PostMatrix(5,3))
fprintf(' $\\mu(\\chi_t=1)$ & Monitoring cost & N & 0.275 & 0.10 & & $%.4f$ & $%.4f$ & $%.4f$ \\\\\n', ...
          PostMatrix(6,1), PostMatrix(6,2), PostMatrix(6,3))
fprintf(' $\\mu(\\chi_t=2)$ & Monitoring cost & N & 0.275 & 0.10 & & $%.4f$ & $%.4f$  & $%.4f$\\\\\n', ...
          PostMatrix(7,1), PostMatrix(7,2), PostMatrix(7,3))     
fprintf(' $\\rho_{\\sigma}(\\chi_t=1)$ & Persistence shock & B & 0.60 & 0.20 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(8,1), PostMatrix(8,2), PostMatrix(8,3))
fprintf(' $\\rho_{\\sigma}(\\chi_t=2)$ & Persistence shock & B & 0.60 & 0.20 & & $%.4f$ & $%.4f$ & $%.4f$ \\\\\n', ...
          PostMatrix(9,1), PostMatrix(9,2), PostMatrix(9,3))
fprintf(' $\\sigma_{\\sigma}(\\chi_t=1)$ & Std Dev shock & Inv-G & 1.00 & 1.00 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(10,1), PostMatrix(10,2), PostMatrix(10,3))
fprintf(' $\\sigma_{\\sigma}(\\chi_t=2)$ & Std Dev shock & Inv-G & 1.00 & 1.00 & & $%.4f$ & $%.4f$ & $%.4f$\\\\\n', ...
          PostMatrix(11,1), PostMatrix(11,2), PostMatrix(11,3))   
fprintf(' \\hline\n')
fprintf('\\end{tabular}\n')
fprintf('\\end{table}\n')
diary off
