%
% Generate Figure 3
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath 'dynare'
addpath '../library'
addpath '../library/Bianchi_methods'
addpath '../library/Bianchi_toolbox'

[Qhat, Ahat, Fhat, Zhat] = load_flat_file('Run_msVAR');

vnames=[{'Panel A. Output'},{'Panel B. VIX'},{'Panel C. Credit spread'}];

nvar  = size(Ahat,1);
m     = size(Fhat,1);
nlag  = (m-1)/nvar;
ns    = size(Ahat,3);

BinDate = datetime('Q4/1962','InputFormat','QQ/yyyy');
EndDate = datetime('Q2/2018','InputFormat','QQ/yyyy');

Date = BinDate:calquarters(1):EndDate;

load('msbvar_data_62q3_18q2.mat');
xdd=[Y VIX BA];
SMM=zeros(size(xdd,1)-1,size(xdd,2)*nlag);
SMM(:,1:3)=xdd(2:end,:);
SMM(:,4:6)=xdd(1:end-1,:);

TTH = zeros(m-1,m,ns);
RQH = zeros(m-1,nvar,ns);

for ir=1:ns
    [BhatCom, C_Com]=fn_companionform(Fhat(:,:,ir)/Ahat(:,:,ir),nvar,nlag);
    TTH(:,:,ir)=[BhatCom,C_Com];
	RQH(1:nvar,:,ir)=permute(inv(sqrt(Zhat(:,:,ir)))*inv(Ahat(:,:,ir)),[2 1]);
end
HHmo=kron(Qhat(:,:,1),Qhat(1:2,1:2,2));

[~,OMEGAH,CSIH] = mss_lom_nonhomog6_note_plus_dsge_RQ6_rev(TTH,RQH,HHmo,0);

co=1;
Tfor=20;
prob1=readmatrix('dynare/smoothed_Run_msVAR.out','FileType','text');
prob=prob1(:,1:end-1); % remove extra column
smbc=prob;
[ MEs, SDs, VARs, COVMs, SECMs ] = cond_moments93_co_pr_rev( OMEGAH, CSIH, ns, Tfor, co, SMM, smbc );

make_it_tight = true;                     % GAP        LOWER UPPER  LEFT RIGHT
subplot = @(m,n,p) subtightplot (m, n, p, [0.12 0.01], [0.08 0.04], [0.05 0.135]);
if ~make_it_tight,  clear subplot;  end

% winter color 
cmap = colormap(winter(length(5:1:Tfor+1)));

figure
for in=1:nvar
    subplot(nvar,1,in)
    for jn=5:1:Tfor+1
        plot(Date,squeeze(real(SDs(in,:,jn))),'color',cmap(jn-4,:))
        hold on
    end
    t=title(vnames{in});
    set(t, 'horizontalAlignment', 'left')
    set(t, 'units', 'normalized')
    h = get(t, 'position');
    set(t, 'position', [0 1 0])
    xlim([Date(1) Date(end)])
    ylim([min(min(squeeze(real(SDs(in,:,5:end))))) max(max(squeeze(real(SDs(in,:,5:end)))))])
    grid on
    box on
    xlabel('years')
    ax = gca;
    ax.YRuler.Exponent = 0;
end
hp4 = get(subplot(3,1,3),'Position');
colormap(cmap)
%colorbar
hcb=colorbar('Position', [hp4(1)+hp4(3)+0.02  hp4(2)  0.05  hp4(2)+hp4(3)*0.98], ...
             'XTickLabel', {'4Q','7Q','10Q','15Q','20Q'}, ...
               'XTick', [0 0.25 0.50 0.75 1]);
%colorTitleHandle = get(hcb,'Title');
%titleString = 'A title';
%set(colorTitleHandle ,'String',titleString);
set(get(hcb,'label'),'string','\fontsize{9}{Horizon}');

set(gcf,'PaperUnits','inches');
set(gcf,'PaperSize',[7 4]);
set(gcf,'PaperPositionMode','manual');
set(gcf,'PaperPosition',[0 0 7 4]);

set(gcf,'renderer','painters');
eval(['print -dpdf Fig2']);
    
    