%
% Generate Figure 2
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath 'dynare'
addpath '../library'

% store results to estimate MS-DSGE
flag_dsge = 0;

% load posterior distribution of IRFs (see recover_IRFs.m)
load IRFs_msbvar_draws.mat

% load coefficients at the mode
[Qhat, Ahat, Fhat, Zhat] = load_flat_file('Run_msVAR');

% choose two regimes
R = [1,4];

bupp1   = 95;   % upper error band (percentile)
bupp    = 84;   % upper error band (percentile)
blow    = 16;   % lower error band (percentile)
blow1   =  5;   % lower error band (percentile)
bmed    = 50;   % impulse response percentile
imstp   = 16+1; % horizon IRFs

t       = 0:imstp-1;
nvar    = size(Ahat,1);
nlag    = (size(Fhat,1)-1)/nvar;
ns      = size(Ahat,3);
ndraws  = size(irf,4);

% memory allocation
irf_quan      = zeros(imstp,nvar,5,ns);
irf_diff_quan = zeros(imstp,nvar,5);

% Get and sort differences in IRFs across regimes
irf_diff      = squeeze(irf(:,:,R(1),:)-irf(:,:,R(2),:));
sort_irf_diff = sort(irf_diff,3);

for idx_ns=1:ns
    % obtain each regime
    irf_to_shock=squeeze(irf(:,:,idx_ns,:));
    
    % sort IRFs
    sort_irf = sort(irf_to_shock,3);
    
    % computing mean, upper and lower bands for IRFs
    irf_quan(:,:,1,idx_ns) = sort_irf(1:imstp,:,round(ndraws*blow1/100));
    irf_quan(:,:,2,idx_ns) = sort_irf(1:imstp,:,round(ndraws*blow/100));
    irf_quan(:,:,3,idx_ns) = sort_irf(1:imstp,:,round(ndraws*bmed/100));
    irf_quan(:,:,4,idx_ns) = sort_irf(1:imstp,:,round(ndraws*bupp/100));
    irf_quan(:,:,5,idx_ns) = sort_irf(1:imstp,:,round(ndraws*bupp1/100));
end

% computing mean, upper and lower bands for IRFs 
irf_diff_quan(:,:,1) = sort_irf_diff(1:imstp,:,round(ndraws*blow1/100));
irf_diff_quan(:,:,2) = sort_irf_diff(1:imstp,:,round(ndraws*blow/100));
irf_diff_quan(:,:,3) = sort_irf_diff(1:imstp,:,round(ndraws*bmed/100));
irf_diff_quan(:,:,4) = sort_irf_diff(1:imstp,:,round(ndraws*bupp/100));
irf_diff_quan(:,:,5) = sort_irf_diff(1:imstp,:,round(ndraws*bupp1/100));

% IRFs
irf_R1_quan = irf_quan(:,:,:,R(1))*100;
irf_R2_quan = irf_quan(:,:,:,R(2))*100;
irf_DI_quan = irf_diff_quan*100;


idx_label1 = 1:nvar:nvar*nvar;
idx_label2 = 2:nvar:nvar*nvar;
idx_label3 = 3:nvar:nvar*nvar;

make_it_tight = true;                     % GAP        LOWER UPPER  LEFT RIGHT
subplot = @(m,n,p) subtightplot (m, n, p, [0.1 0.08], [0.06 0.03], [0.08 0.012]);
if ~make_it_tight,  clear subplot;  end

figure
for i=1:nvar
subplot(nvar,3,idx_label1(i))
plot(t,zeros(imstp,1),'color','k','LineStyle',':','LineWidth',2.5)
hold on
plot(t,irf_R1_quan(:,i,1),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,2),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,3),'LineStyle','-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,4),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,5),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
ylim([ min([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ...
max([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ])
xlim([t(1) t(end)])
box on
grid on
set(gca,'xtick',[0 2 4 8 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
if i==1
    ylabel({'Percent'})
    %ylabel({'Percent Deviation','from Steady State'})
else
    ylabel('Percentage Points')
end
xlabel('quarters after shock')

if i==1
	tt=title('Panel A. Output [1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==2
    tt=title('Panel B. VIX [1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==3
    tt=title('Panel C. Credit spread [1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

subplot(nvar,3,idx_label2(i))
plot(t,zeros(imstp,1),'color','k','LineStyle',':','LineWidth',2.5)
hold on
plot(t,irf_R2_quan(:,i,1),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,2),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,3),'LineStyle','-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,4),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,5),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
ylim([ min([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ...
max([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ])
xlim([t(1) t(end)])
box on
grid on
set(gca,'xtick',[0 2 4 8 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
if i==1
    ylabel({'Percent'})
    %ylabel({'Percent Deviation','from Steady State'})
else
    ylabel('Percentage Points')
end
xlabel('quarters after shock')

if i==1
	tt=title('Panel D. Output [2]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==2
    tt=title('Panel E. VIX [2]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==3
    tt=title('Panel F. Credit spread [2]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

subplot(nvar,3,idx_label3(i))
plot(t,zeros(imstp,1),'color','k','LineStyle',':','LineWidth',2.5)
hold on
plot(t,irf_DI_quan(:,i,1),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_DI_quan(:,i,2),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
aaa=plot(t,irf_DI_quan(:,i,3),'LineStyle','-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
aha=plot(t,irf_DI_quan(:,i,4),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
ahh=plot(t,irf_DI_quan(:,i,5),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
ylim([ min([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ...
max([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ])
xlim([t(1) t(end)])
box on
grid on
set(gca,'xtick',[0 2 4 8 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
ylabel('Percentage Points')
xlabel('quarters after shock')

if i==1
	tt=title('Panel G. Output [2]-[1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==2
    tt=title('Panel H. VIX [2]-[1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==3
    tt=title('Panel I. Credit spread [2]-[1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

end
legend([aaa,aha,ahh],'Median','68% error bands','90% error bands','Location','NorthEast');

set(gcf,'PaperUnits','inches');
set(gcf,'PaperSize',[8 8]);
set(gcf,'PaperPositionMode','manual');
set(gcf,'PaperPosition',[0 0 8 8]);

set(gcf,'renderer','painters');
eval(['print -dpdf Fig3']);

if (flag_dsge)
    % mean
    IRFu        = zeros(imstp,nvar,2);
    IRFu(:,:,1) = mean(squeeze(irf(1:imstp,:,R(1),:)*100),3);
    IRFu(:,:,2) = mean(squeeze(irf(1:imstp,:,R(2),:)*100),3);
    
    % std
    IRFuSE        = zeros(imstp,nvar,2);
    IRFuSE(:,:,1) = std(squeeze(irf(1:imstp,:,R(1),:)*100),[],3);
    IRFuSE(:,:,2) = std(squeeze(irf(1:imstp,:,R(2),:)*100),[],3);
    
    % obtain percentiles
    irf_quan_68_90 = irf_quan(:,:,:,R)*100;
    
    % save
    save MSVAR_IRFsAndSEs IRFu IRFuSE irf_quan_68_90
end