%
% Generate Tables 1 and 2
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

load('IRFs_msbvar_draws','Z','Q');

[Qhat, Ahat, Fhat, Zhat] = load_flat_file('Run_msVAR');

bupp     = 84;    % upper error band (percentile)
blow     = 16;    % lower error band (percentile)
bmed     = 50;    % impulse response percentile

ndraws   = size(Q,4);

z_11_hat   = 1./sqrt(diag(Zhat(:,:,1)));
z_22_hat   = 1./sqrt(diag(Zhat(:,:,3)));
z_33_hat   = 1./sqrt(diag(Zhat(:,:,5)));

q11_v = squeeze(Q(1,1,1,1:ndraws));
q22_v = squeeze(Q(2,2,1,1:ndraws));
q33_v = squeeze(Q(3,3,1,1:ndraws));

q11_c = squeeze(Q(1,1,2,1:ndraws));
q22_c = squeeze(Q(2,2,2,1:ndraws));

z_11   = 1./sqrt(squeeze(Z(1,1,[1,3,5],1:ndraws)));
z_22   = 1./sqrt(squeeze(Z(2,2,[1,3,5],1:ndraws)));
z_33   = 1./sqrt(squeeze(Z(3,3,[1,3,5],1:ndraws)));

% memory allocation
q11_v_quan = zeros(3,1);
q22_v_quan = zeros(3,1);
q33_v_quan = zeros(3,1);

q11_c_quan = zeros(3,1);
q22_c_quan = zeros(3,1);

z_11_quan   = zeros(3,3);
z_22_quan   = zeros(3,3);
z_33_quan   = zeros(3,3);

% sort 
sort_q11_v = sort(q11_v);
sort_q22_v = sort(q22_v);
sort_q33_v = sort(q33_v);
%
sort_q11_c = sort(q11_c);
sort_q22_c = sort(q22_c);
%
sort_z_11 = sort(z_11,2);
sort_z_22 = sort(z_22,2);
sort_z_33 = sort(z_33,2);
    
% computing mean, upper and lower bands for HDs 
q11_v_quan(1) = sort_q11_v(round(ndraws*blow/100));
q11_v_quan(2) = sort_q11_v(round(ndraws*bmed/100));
q11_v_quan(3) = sort_q11_v(round(ndraws*bupp/100));
%
q22_v_quan(1) = sort_q22_v(round(ndraws*blow/100));
q22_v_quan(2) = sort_q22_v(round(ndraws*bmed/100));
q22_v_quan(3) = sort_q22_v(round(ndraws*bupp/100));
%
q33_v_quan(1) = sort_q33_v(round(ndraws*blow/100));
q33_v_quan(2) = sort_q33_v(round(ndraws*bmed/100));
q33_v_quan(3) = sort_q33_v(round(ndraws*bupp/100));
%
q11_c_quan(1) = sort_q11_c(round(ndraws*blow/100));
q11_c_quan(2) = sort_q11_c(round(ndraws*bmed/100));
q11_c_quan(3) = sort_q11_c(round(ndraws*bupp/100));
%
q22_c_quan(1) = sort_q22_c(round(ndraws*blow/100));
q22_c_quan(2) = sort_q22_c(round(ndraws*bmed/100));
q22_c_quan(3) = sort_q22_c(round(ndraws*bupp/100));
%
z_11_quan(:,1) = sort_z_11(:,round(ndraws*blow/100));
z_11_quan(:,2) = sort_z_11(:,round(ndraws*bmed/100));
z_11_quan(:,3) = sort_z_11(:,round(ndraws*bupp/100));
%
z_22_quan(:,1) = sort_z_22(:,round(ndraws*blow/100));
z_22_quan(:,2) = sort_z_22(:,round(ndraws*bmed/100));
z_22_quan(:,3) = sort_z_22(:,round(ndraws*bupp/100));
%
z_33_quan(:,1) = sort_z_33(:,round(ndraws*blow/100));
z_33_quan(:,2) = sort_z_33(:,round(ndraws*bmed/100));
z_33_quan(:,3) = sort_z_33(:,round(ndraws*bupp/100));

%
fprintf('\\begin{table}\n')
fprintf('\\begin{threeparttable}\n')
fprintf('\\center\n')
fprintf('\\caption{Relative shock standard deviations across regimes}\n')
fprintf('\\label{residual_2c2v}\n')
fprintf('\\begin{tabular}{cccc}\n')
fprintf(' \\hline\n ')
fprintf(' \\hline\n')
fprintf(' & Production gdp  & Uncertainty vix &  Financial sp \\\\\n')
fprintf(' \\hline\n')
fprintf(' $s_t^{\\text{v}}=1$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ \\\\\n', ...
          z_11_quan(1,1), z_11_quan(1,3), z_11_hat(1),  ...
          z_22_quan(1,1), z_22_quan(1,3), z_11_hat(2),  ...
          z_33_quan(1,1), z_33_quan(1,3), z_11_hat(3))
fprintf(' $s_t^{\\text{v}}=2$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ \\\\\n', ...
          z_11_quan(2,1), z_11_quan(2,3), z_22_hat(1),  ...
          z_22_quan(2,1), z_22_quan(2,3), z_22_hat(2),  ...
          z_33_quan(2,1), z_33_quan(2,3), z_22_hat(3))
fprintf(' $s_t^{\\text{v}}=3$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ \\\\\n', ...
          z_11_quan(3,1), z_11_quan(3,3), z_33_hat(1),  ...
          z_22_quan(3,1), z_22_quan(3,3), z_33_hat(2),  ...
          z_33_quan(3,1), z_33_quan(3,3), z_33_hat(3))
fprintf(' \\hline\n')
fprintf('\\end{tabular}\n')
fprintf('\\begin{tablenotes}[para,flushleft]\n')
fprintf('\\small\n')
fprintf('\\textit{Note:} Posterior modes and [16th , 64th] percentiles are reported.\n')
fprintf('\\end{tablenotes}\n')
fprintf('\\end{threeparttable}\n')
fprintf('\\end{table}\n')

%
fprintf('\\begin{table}\n')
fprintf('\\begin{threeparttable}\n')
fprintf('\\center\n')
fprintf('\\caption{Estimated transition matrices}\n')
fprintf('\\label{tab_H_dist}\n')
fprintf('\\begin{tabular}{cccccccc}\n')
fprintf(' \\hline\n ')
fprintf(' \\hline\n')
fprintf(' \\multicolumn{4}{c}{VAR disturbances} & & \\multicolumn{3}{c}{VAR coefficients} \\\\\n')
fprintf(' \\cline{1-4}\n')
fprintf(' \\cline{6-8}\n')
fprintf('  &  $s_t^{\\text{v}}=1$ &  $s_t^{\\text{v}}=2$ &  $s_t^{\\text{v}}=3$ & & & $s_t^{\\text{c}}=1$ &  $s_t^{\\text{c}}=2$ \\\\\n')
fprintf(' $s_t^{\\text{v}}=1$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & & $s_t^{\\text{c}}=1$  & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ \\\\\n', ...
          q11_v_quan(1), q11_v_quan(3), Qhat(1,1,1),  ...
          (1-q22_v_quan(3))/2, (1-q22_v_quan(1))/2, (1-Qhat(2,2,1))/2,  ...
          0.0000,0.0000,0.000, ...
          q11_c_quan(1), q11_c_quan(3), Qhat(1,1,2), ...
          1-q22_c_quan(3), 1-q22_c_quan(1), 1-Qhat(2,2,2))
fprintf(' $s_t^{\\text{v}}=2$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & & $s_t^{\\text{c}}=2$  & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ \\\\\n', ...
          1-q11_v_quan(3), 1-q11_v_quan(1), 1-Qhat(1,1,1),  ...
          q22_v_quan(1), q22_v_quan(3), Qhat(2,2,1),  ...
          1-q33_v_quan(3), 1-q33_v_quan(1), 1-Qhat(3,3,1), ...
          1-q11_c_quan(3), 1-q11_c_quan(1), 1-Qhat(1,1,2), ...
          q22_c_quan(1), q22_c_quan(3), Qhat(2,2,2))
      
fprintf(' $s_t^{\\text{v}}=3$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & $\\underset{[%.4f ; %.4f ]}{%.4f}$ & &   &  & \\\\\n', ...
          0.0000,0.0000,0.000, ...
          (1-q22_v_quan(3))/2, (1-q22_v_quan(1))/2, (1-Qhat(2,2,1))/2,  ...
          q33_v_quan(1), q33_v_quan(3), Qhat(3,3,1))

fprintf(' \\hline\n')
fprintf('\\end{tabular}\n')
fprintf('\\begin{tablenotes}[para,flushleft]\n')
fprintf('\\small\n')
fprintf('\\textit{Note:} Posterior modes and [16th , 64th] percentiles are reported.\n')
fprintf('\\end{tablenotes}\n')
fprintf('\\end{threeparttable}\n')
fprintf('\\end{table}\n')



% $s_t^{\text{v}}=1$ & $\underset{[0.9153 ; 0.9709 ]}{0.9432}$ & $\underset{[0.0568 ; 0.1727 ]}{0.1828}$ & $\underset{[0.0000 ; 0.0000 ]}{0.0000}$  & & $s_t^{\text{c}}=1$ & $\underset{[0.4522 ; 0.8752 ]}{0.8969}$ & $\underset{[0.0114 ; 0.1088 ]}{0.0676}$ \\
% $s_t^{\text{v}}=2$ & $\underset{[0.0291 ; 0.0847]}{0.0568}$ & $\underset{[0.6546 ; 0.8864 ]}{0.6345}$ & $\underset{[0.0211 ; 0.1056 ]}{0.0154}$ & & $s_t^{\text{c}}=2$ & $\underset{[0.1248 ; 0.5477 ]}{0.1031}$ & $\underset{[0.8912 ; 0.9886 ]}{0.9324}$ \\
% $s_t^{\text{v}}=3$ & $\underset{[0.0000 ; 0.0000 ]}{0.0000}$ & $\underset{[0.0568 ; 0.1727 ]}{0.1828}$ & $\underset{[0.8944 ; 0.9789 ]}{0.9846}$ & & &  &  \\



