%
% Recover IRFs from MS-SVAR model
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath(genpath('../library'));

% load coefficients 
load load_draws_msVAR.mat

% load data sample
load dynare/msbvar_data_62q3_18q2.mat
xdd = [Y VIX BA];

bupp    = 84;    % upper error band (percentile)
blow    = 16;    % lower error band (percentile)
bmed    = 50;    % impulse response percentile
nexo    = 1;     % constant
imstp   = 20;    % IRFs horizon
M       = 2;     % position of the structural shock

nstates = size(Agibbs,3);                        % # regimes governing equation coefficients
nvar    = size(Agibbs,1);                        % # variables
nlags   = (size(Fgibbs,1)-nexo)/size(Fgibbs,2);  % # lags
ndraws  = size(Agibbs,4);                        % # draws
nproc   = 2;

% impulse response functions
irf = zeros(imstp,nvar,nstates,ndraws);
%
rootsinv_store = zeros(size(Fgibbs(:,:,:,1),1)-1,2);
%
cc = [];

Anew = zeros(nvar,nvar,nstates,ndraws);
Bnew = zeros(nvar*nlags+nexo,nvar,nstates,ndraws);
%Znew = zeros(nvar,nvar,nstates,ndraws);
%Qnew = zeros(nstates/nproc,nstates/nproc,nproc,ndraws);

for idx_draws=1:ndraws
    %
    imf3gibbs=zeros(imstp,nvar,nvar,nstates);
    idx_tmp=zeros(2,1);
    %
    fprintf(1,'\nSimulation step %2.0f of %2.0f',[idx_draws,ndraws])
    for idx_ns=1:nstates
        %
        Bgibbs=Fgibbs(:,:,idx_ns,idx_draws)/Agibbs(:,:,idx_ns,idx_draws);
        %
        swish=inv(Agibbs(:,:,idx_ns,idx_draws)*sqrt(Zgibbs(:,:,idx_ns,idx_draws)));
        %
        [Q,idx_tmp(idx_ns)]=SRestrictRWZalg(swish,Bgibbs,nvar,nlags,1);
        %
        swish1=Q*swish;
        %
        Anew(:,:,idx_ns,idx_draws) = (Agibbs(:,:,idx_ns,idx_draws)*sqrt(Zgibbs(:,:,idx_ns,idx_draws)))*Q';
        Bnew(:,:,idx_ns,idx_draws) = Bgibbs;
        %
        imfgibbs = fn_impulse(Bgibbs,swish1,[nvar nlags imstp]);
        % imf3: row--steps, column--nvar responses, 3rd dimension--nvar shocks		
        imf3gibbs(:,:,:,idx_ns) = reshape(imfgibbs,size(imfgibbs,1),nvar,nvar);
        % normalize VIX shock at a 10 pp VIX increase
        Mult = 0.10/swish1(M,M);
        ii = 1;
        for ii=1:nvar
            irf(:,ii,idx_ns,idx_draws) = Mult*imf3gibbs(:,ii,M,idx_ns);
            ii = ii+1;
        end
        %
        rootsinv = sort(abs(fn_varoots(Bgibbs,nvar,nlags)));
        %
        rootsinv_store(:,idx_ns) = rootsinv;
        %
        clear swish Bgibbs imfgibbs Mult
    end
    
	% check if regime switching occurs in the sample
    pr_sT = get_prob_from_coeffs(xdd,Agibbs(:,:,:,idx_draws),Fgibbs(:,:,:,idx_draws),Zgibbs(:,:,:,idx_draws),kron(Qgibbs(:,:,1,idx_draws),Qgibbs(1:2,1:2,2,idx_draws)));
    sd    = (pr_sT(:,1)+pr_sT(:,3)+pr_sT(:,5))>0.5;

    if sum(sd==1)<=2 || sum(sd==0)<=2 
        tmp=0;
    else
        tmp=1;
    end
    
    if (rootsinv_store(end,1) <=1.01) && (rootsinv_store(end,2) <=1.01) && (sum(idx_tmp)==size(idx_tmp,1)) && (tmp==1) % avoid explosive paths
        cc = [cc idx_draws];
    end
end

irf    = irf(:,:,:,cc');
Z      = Zgibbs(:,:,:,cc');
Q      = Qgibbs(:,:,:,cc');


save IRFs_msbvar_draws irf Z Q
