%
% Recover IRFs from MS-SVAR model [MaxVDs identification]
%
% by Stephane LHUISSIER and Fabien TRIPIER

clear all; clc;

addpath(genpath('../../library'))

% load coefficients 
load ../load_draws_msVAR.mat

% load data sample
load ../dynare/msbvar_data_62q3_18q2.mat
xdd = [Y VIX BA];

bupp    = 84;    % upper error band (percentile)
blow    = 16;    % lower error band (percentile)
bmed    = 50;    % impulse response percentile
bupp1   = 95;    % upper error band (percentile)
blow1   = 5;     % lower error band (percentile)
nexo    = 1;     % constant
imstp   = 16+1;  % IRFs horizon
M       = 2;     % position of the structural shock
thres   = 60;    % 
R       = [1,4]; %

t       = 0:imstp-1;
nstates = size(Agibbs,3);                        % # regimes governing equation coefficients
nvar    = size(Agibbs,1);                        % # variables
nlags   = (size(Fgibbs,1)-nexo)/size(Fgibbs,2);  % # lags
ndraws  = size(Agibbs,4);                        % # draws

% impulse response functions
irf = zeros(imstp,nvar,nstates,ndraws);
%
rootsinv_store = zeros(size(Fgibbs(:,:,:,1),1)-1,2);
%
cc = [];

for idx_draws=1:ndraws
    %
    imf3gibbs=zeros(imstp,nvar,nvar,nstates);
    idx_tmp=zeros(2,1);
    %
    fprintf(1,'Simulation step %2.0f of %2.0f\n',[idx_draws,ndraws])
    for idx_ns=1:nstates
        %
        Bgibbs=Fgibbs(:,:,idx_ns,idx_draws)/Agibbs(:,:,idx_ns,idx_draws);
        %
        swish=inv(Agibbs(:,:,idx_ns,idx_draws)*sqrt(Zgibbs(:,:,idx_ns,idx_draws)));
        %
        [Q,idx_tmp(idx_ns)]=SRestrictRWZalg_MaxVDs(swish,Bgibbs,nvar,nlags,1);
        %
        swish1=Q*swish;
        %
        imfgibbs = fn_impulse(Bgibbs,swish1,[nvar nlags imstp]);
        % imf3: row--steps, column--nvar responses, 3rd dimension--nvar shocks		
        imf3gibbs(:,:,:,idx_ns) = reshape(imfgibbs,size(imfgibbs,1),nvar,nvar);
        % normalize VIX shock at a 10 pp VIX increase
        Mult = 0.10/swish1(M,M);
        ii = 1;
        for ii=1:nvar
            irf(:,ii,idx_ns,idx_draws) = Mult*imf3gibbs(:,ii,M,idx_ns);
            ii = ii+1;
        end
        %
        rootsinv = sort(abs(fn_varoots(Bgibbs,nvar,nlags)));
        %
        rootsinv_store(:,idx_ns) = rootsinv;
        %
        clear swish Bgibbs imfgibbs Mult
    end
    
	% check if regime switching occurs in the sample
    pr_sT = get_prob_from_coeffs(xdd,Agibbs(:,:,:,idx_draws),Fgibbs(:,:,:,idx_draws),Zgibbs(:,:,:,idx_draws),kron(Qgibbs(:,:,1,idx_draws),Qgibbs(1:2,1:2,2,idx_draws)));
    sd    = (pr_sT(:,1)+pr_sT(:,3)+pr_sT(:,5))>0.5;

    if sum(sd==1)<=2 || sum(sd==0)<=2 
        tmp=0;
    else
        tmp=1;
    end
    
    if (rootsinv_store(end,1) <=1.01) && (rootsinv_store(end,2) <=1.01) && (sum(idx_tmp)==size(idx_tmp,1)) && (tmp==1) % avoid explosive paths
        cc = [cc idx_draws];
    end
end

irf    = irf(:,:,:,cc');
ndraws = size(cc,2);

% memory allocation
irf_quan      = zeros(imstp,nvar,5,nstates);
irf_diff_quan = zeros(imstp,nvar,5);

% Get and sort differences in IRFs across regimes
irf_diff      = squeeze(irf(:,:,R(1),:)-irf(:,:,R(2),:));
sort_irf_diff = sort(irf_diff,3);

for idx_ns=1:nstates
    % obtain each regime
    irf_to_shock=squeeze(irf(:,:,idx_ns,:));
    
    % sort IRFs
    sort_irf = sort(irf_to_shock,3);
    
    % computing mean, upper and lower bands for IRFs
    irf_quan(:,:,1,idx_ns) = sort_irf(1:imstp,:,round(ndraws*blow1/100));
    irf_quan(:,:,2,idx_ns) = sort_irf(1:imstp,:,round(ndraws*blow/100));
    irf_quan(:,:,3,idx_ns) = sort_irf(1:imstp,:,round(ndraws*bmed/100));
    irf_quan(:,:,4,idx_ns) = sort_irf(1:imstp,:,round(ndraws*bupp/100));
    irf_quan(:,:,5,idx_ns) = sort_irf(1:imstp,:,round(ndraws*bupp1/100));
end

% computing mean, upper and lower bands for IRFs 
irf_diff_quan(:,:,1) = sort_irf_diff(1:imstp,:,round(ndraws*blow1/100));
irf_diff_quan(:,:,2) = sort_irf_diff(1:imstp,:,round(ndraws*blow/100));
irf_diff_quan(:,:,3) = sort_irf_diff(1:imstp,:,round(ndraws*bmed/100));
irf_diff_quan(:,:,4) = sort_irf_diff(1:imstp,:,round(ndraws*bupp/100));
irf_diff_quan(:,:,5) = sort_irf_diff(1:imstp,:,round(ndraws*bupp1/100));

% IRFs
irf_R1_quan = irf_quan(:,:,:,R(1))*100;
irf_R2_quan = irf_quan(:,:,:,R(2))*100;
irf_DI_quan = irf_diff_quan*100;


idx_label1 = 1:nvar:nvar*nvar;
idx_label2 = 2:nvar:nvar*nvar;
idx_label3 = 3:nvar:nvar*nvar;

make_it_tight = true;                     % GAP        LOWER UPPER  LEFT RIGHT
subplot = @(m,n,p) subtightplot (m, n, p, [0.1 0.08], [0.06 0.03], [0.08 0.012]);
if ~make_it_tight,  clear subplot;  end

figure
for i=1:nvar
subplot(nvar,3,idx_label1(i))
plot(t,zeros(imstp,1),'color','k','LineStyle',':','LineWidth',2.5)
hold on
plot(t,irf_R1_quan(:,i,1),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,2),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,3),'LineStyle','-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,4),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R1_quan(:,i,5),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
ylim([ min([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ...
max([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ])
xlim([t(1) t(end)])
box on
grid on
set(gca,'xtick',[0 2 4 8 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
if i==1
    ylabel({'Percent'})
    %ylabel({'Percent Deviation','from Steady State'})
else
    ylabel('Percentage Points')
end
xlabel('quarters after shock')

if i==1
	tt=title('Panel A. Output [1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==2
    tt=title('Panel B. VIX [1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==3
    tt=title('Panel C. Credit spread [1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

subplot(nvar,3,idx_label2(i))
plot(t,zeros(imstp,1),'color','k','LineStyle',':','LineWidth',2.5)
hold on
plot(t,irf_R2_quan(:,i,1),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,2),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,3),'LineStyle','-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,4),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_R2_quan(:,i,5),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
ylim([ min([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ...
max([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ])
xlim([t(1) t(end)])
box on
grid on
set(gca,'xtick',[0 2 4 8 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
if i==1
    ylabel({'Percent'})
    %ylabel({'Percent Deviation','from Steady State'})
else
    ylabel('Percentage Points')
end
xlabel('quarters after shock')

if i==1
	tt=title('Panel D. Output [2]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==2
    tt=title('Panel E. VIX [2]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==3
    tt=title('Panel F. Credit spread [2]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

subplot(nvar,3,idx_label3(i))
plot(t,zeros(imstp,1),'color','k','LineStyle',':','LineWidth',2.5)
hold on
plot(t,irf_DI_quan(:,i,1),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
plot(t,irf_DI_quan(:,i,2),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
aaa=plot(t,irf_DI_quan(:,i,3),'LineStyle','-','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
aha=plot(t,irf_DI_quan(:,i,4),'LineStyle','-.','color',[0 0.4470 0.7410],'LineWidth',1.0);
hold on
ahh=plot(t,irf_DI_quan(:,i,5),'LineStyle',':','color',[0 0.4470 0.7410],'LineWidth',1.0);
ylim([ min([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ...
max([irf_R1_quan(:,i,1);irf_R1_quan(:,i,5);irf_R2_quan(:,i,1);irf_R2_quan(:,i,5);irf_DI_quan(:,i,1);irf_DI_quan(:,i,5)]) ])
xlim([t(1) t(end)])
box on
grid on
set(gca,'xtick',[0 2 4 8 16])
set(gca,'xgrid','on') 
set(gca,'ygrid','on')
ylabel('Percentage Points')
xlabel('quarters after shock')

if i==1
	tt=title('Panel G. Output [2]-[1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==2
    tt=title('Panel H. VIX [2]-[1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

if i==3
    tt=title('Panel I. Credit spread [2]-[1]');
    set(tt, 'horizontalAlignment', 'left')
    set(tt, 'units', 'normalized')
    h = get(tt, 'position');
    set(tt, 'position', [0 1 0])
end

end
legend([aaa,aha,ahh],'Median','68% error bands','90% error bands','Location','NorthEast');

set(gcf,'PaperUnits','inches');
set(gcf,'PaperSize',[8 8]);
set(gcf,'PaperPositionMode','manual');
set(gcf,'PaperPosition',[0 0 8 8]);

set(gcf,'renderer','painters');
eval(['print -dpdf Fig_MaxVDs']);
